package org.greenstone.gsdl3.comms;

import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.core.*;

// classes needed for SOAP stuff
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Vector;
import org.apache.soap.SOAPException;
import org.apache.soap.Fault;
import org.apache.soap.Constants;
import org.apache.soap.rpc.Call;
import org.apache.soap.rpc.Parameter;
import org.apache.soap.rpc.Response;

import org.w3c.dom.Node; 
import org.w3c.dom.NodeList; 
import org.w3c.dom.Document; 
import org.w3c.dom.Element; 

public class SOAPCommunicator
    extends Communicator {

    /** the call object that does the SOAP talking */
    private Call call_ = null;
    
    public SOAPCommunicator() {
	
	call_ = new Call();
	// Set Encoding Style to standard SOAP encoding
	call_.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
    }
    
    public void setConnectionName(String name) {
	System.out.println("name="+name);
	connection_name_ = name;
	// Set Object URI and Method Name
	call_.setTargetObjectURI (name);
	call_.setMethodName ("process"); // everything uses process
    }
    
    public boolean configure() {
	return true;
    }


    // put this into Communicator??
    public String process(String in) {
	// needs to catch SOAPException, MalformedURLException
	try {
	    // add the message encloser back on, and edit the to field
	    Document doc = converter_.getDOM(in);
	    Document newdoc = converter_.newDOM();
	    Element top = newdoc.createElement("message");
	    top.appendChild(newdoc.importNode(doc.getDocumentElement(), true));
	    //newdoc.appendChild(top);
	    NodeList requests = top.getElementsByTagName("request");
	    if (requests.getLength()==0) {
		// no requests
		return "";
	    }
	    for (int i=0;i<requests.getLength(); i++) {
		Element e = (Element)requests.item(i);
		String to = e.getAttribute("to");
		to = GSPath.removeFirstLink(to); // remove the server name, should check that it is present
		e.setAttribute("to", to);
		String from = e.getAttribute("from");
		from = GSPath.appendLink(from, local_site_name_);
		e.setAttribute("from", from);
	    }

	    String converted_request = converter_.getString(top);
	    System.out.println("converted request = "+converted_request);
	    //  Set Method Parameters
	    Parameter param = new Parameter("xml_in", String.class,
					    converted_request, Constants.NS_URI_SOAP_ENC);
	    Vector param_list = new Vector ();
	    param_list.addElement (param);
	    call_.setParams (param_list);
	    
	    //  Set the URL for the Web Service
	    URL url = new URL(connection_address_);
	    
	    // Invoke the Service
	    Response resp = call_.invoke (url, "");
	    
	    // Check for Faults
	    if (!resp.generatedFault()) {
	    // Extract Return value
		Parameter result = resp.getReturnValue ();
		String res = (String) result.getValue();
		return res;
	    }
	    else {
		//  Extract Fault Code and String
		Fault f = resp.getFault();
		String faultCode = f.getFaultCode();
		String faultString = f.getFaultString();
		System.err.println("Fault Occurred (details follow):");
		System.err.println("Fault Code:  "+faultCode);
		System.err.println("Fault String:  "+faultString);
		return new String ("<message>Fault Occurred.  No greeting for you!</message>");
	    }
	} catch (SOAPException e) {
	    System.out.println("SOAP error:"+e.getMessage());
	    return "";
	} catch (MalformedURLException e) {
	    System.out.println("URL error:"+e.getMessage());
	    return "";
	}
    }
    
    public Node process(Node xml_in) {
	String request = converter_.getString(xml_in);
	String result = process(request);
	return converter_.getDOM(result).getDocumentElement();
	
    }
}








